/*
 * shohaku Copyright (C) 2005 tomoya nagatani
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */
package shohaku.core.beans;

/**
 * メソッドの引数情報のリストを格納する機能を提供します。
 */
public class ArgumentsInfo extends FeatureInfo {

    /* 引数の型。 */
    private Class[] parameterTypes;

    /* 引数の値。 */
    private Object[] parameterValues;

    /* 引数情報。 */
    private ArgumentInfo[] parameteres;

    /**
     * デフォルトコンストラクタ。
     */
    public ArgumentsInfo() {
        // no op
    }

    /**
     * 引数情報の配列を指定して初期化します。
     * 
     * @param params
     *            引数の情報
     */
    public ArgumentsInfo(ArgumentInfo[] params) {
        setParameteres(params);
    }

    /**
     * 引数の型と値を指定して初期化します。
     * 
     * @param type
     *            引数の型
     * @param value
     *            引数の値
     */
    public ArgumentsInfo(Class type, Object value) {
        setParameteres(new Class[] { type }, new Object[] { value });
    }

    /**
     * 引数の型と値を指定して初期化します。
     * 
     * @param types
     *            引数の型
     * @param values
     *            引数の値
     */
    public ArgumentsInfo(Class[] types, Object[] values) {
        setParameteres(types, values);
    }

    /*
     * Property
     */

    /**
     * 引数の値を返却します。
     * 
     * @return 引数の値
     */
    public Object[] getParameterValues() {
        return parameterValues;
    }

    /**
     * 引数の型を返却します。
     * 
     * @return 引数の型
     */
    public Class[] getParameterTypes() {
        return parameterTypes;
    }

    /**
     * 引数情報の配列を返却します。
     * 
     * @return 引数情報の配列
     */
    public ArgumentInfo[] getParameteres() {
        return parameteres;
    }

    /**
     * 引数情報の配列を格納します。
     * 
     * @param params
     *            引数情報の配列
     */
    public void setParameteres(ArgumentInfo[] params) {
        if (params == null) {
            throw new NullPointerException("argument is null.");
        }
        Class[] types = new Class[params.length];
        Object[] values = new Object[params.length];
        for (int i = 0; i < params.length; i++) {
            if (params[i].getType() == null) {
                throw new NullPointerException("argument params[i].types is null.");
            }
            types[i] = params[i].getType();
            values[i] = params[i].getValue();
        }
        this.parameterTypes = types;
        this.parameterValues = values;
        this.parameteres = params;
    }

    /**
     * 引数の型と値を格納します。
     * 
     * @param types
     *            引数の型
     * @param values
     *            引数の値
     */
    public void setParameteres(Class[] types, Object[] values) {
        if (types == null || values == null) {
            throw new NullPointerException("argument is null.");
        }
        if (types.length != values.length) {
            throw new IllegalArgumentException("types.length != values.length.");
        }

        ArgumentInfo[] params = new ArgumentInfo[types.length];
        for (int i = 0; i < params.length; i++) {
            if (types[i] == null) {
                throw new NullPointerException("argument params[i].types is null.");
            }
            params[i] = new ArgumentInfo(types[i], values[i]);
        }
        this.parameterTypes = types;
        this.parameterValues = values;
        this.parameteres = params;
    }
}
