/*
 * shohaku Copyright (C) 2005 tomoya nagatani
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */
package shohaku.core.beans;

import java.beans.BeanInfo;
import java.beans.IntrospectionException;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.lang.reflect.Array;
import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import shohaku.core.helpers.Cutter;
import shohaku.core.helpers.Eval;
import shohaku.core.helpers.Seek;
import shohaku.core.lang.Boxing;
import shohaku.core.lang.ObjectCreationException;
import shohaku.core.text.Format;
import shohaku.core.util.cel.CELBinder;
import shohaku.core.util.cel.CELFormatException;
import shohaku.core.util.cel.CELHelper;
import shohaku.core.util.cel.CELReturnValue;

/**
 * <code>JavaBean</code>を制御するユーティリティを提供します。 <br>
 * <br>
 * ライブラリの依存性を最小化する方針から<code>Jakarta</code>の<code>BeanUtils</code>等の高機能ライブラリは使用しません。 <br>
 * コアライブラリには利用頻度の高い機能のみを定義し、必要に応じて他のライブラリを使用します。
 */
public class BeanUtilities {

    /*
     * PropertyDescriptor
     */

    /**
     * ビーンの<code>java.beans.PropertyDescriptor</code>を返却します。
     * 
     * @param bean
     *            ビーンのインスタンス
     * @return ビーンの<code>java.beans.PropertyDescriptor</code>
     */
    public static PropertyDescriptor[] getPropertyDescriptors(Object bean) {
        if (bean == null) {
            throw new NullPointerException("argument is null.");
        }
        return (getPropertyDescriptors(bean.getClass()));
    }

    /**
     * ビーンの<code>java.beans.PropertyDescriptor</code>を返却します。
     * 
     * @param beanClass
     *            ビーンのクラス
     * @return ビーンの<code>java.beans.PropertyDescriptor</code>
     */
    public static PropertyDescriptor[] getPropertyDescriptors(Class beanClass) {
        if (beanClass == null) {
            throw new NullPointerException("argument is null.");
        }
        PropertyDescriptor[] descriptors = null;
        BeanInfo beanInfo = null;
        try {
            beanInfo = Introspector.getBeanInfo(beanClass);
        } catch (IntrospectionException e) {
            return new PropertyDescriptor[0];
        }
        descriptors = beanInfo.getPropertyDescriptors();
        if (descriptors == null) {
            descriptors = new PropertyDescriptor[0];
        }
        return descriptors;
    }

    /*
     * Property
     */

    /**
     * ビーンのプロパティをマップに格納して返却します。
     * 
     * @param bean
     *            ビーンのインスタンス
     * @return 全てのプロパティを格納するマップ
     * @throws InvocationBeansException
     *             プロパティの呼出に失敗した場合
     */
    public static Map getProperties(Object bean) throws InvocationBeansException {

        if (bean == null) {
            throw new NullPointerException("argument is null.");
        }
        try {

            Map props = new HashMap();
            PropertyDescriptor[] ds = getPropertyDescriptors(bean);
            for (int i = 0; i < ds.length; i++) {
                Method method = ds[i].getReadMethod();
                if (method != null) {
                    props.put(ds[i].getName(), method.invoke(bean, null));
                }
            }
            return props;

        } catch (IllegalArgumentException e) {
            throw new InvocationBeansException(e);
        } catch (IllegalAccessException e) {
            throw new InvocationBeansException(e);
        } catch (InvocationTargetException e) {
            throw new InvocationBeansException(e);
        }

    }

    /**
     * ビーンのプロパティをマップに格納して返却します。
     * 
     * @param bean
     *            ビーンのインスタンス
     * @param props
     *            プロパティを格納するマップ
     * @return 引数のビーンのインスタンスの参照
     * @throws InvocationBeansException
     *             プロパティの呼出に失敗した場合
     */
    public static Object setProperties(Object bean, Map props) throws InvocationBeansException {

        if (bean == null) {
            throw new NullPointerException("argument is null.");
        }
        try {

            Map propsDesc = getMapPropertyDescriptors(bean);
            for (Iterator i = props.entrySet().iterator(); i.hasNext();) {
                Map.Entry e = (Map.Entry) i.next();
                String name = (String) e.getKey();// Not String throw ClassCastException
                PropertyDescriptor pd = (PropertyDescriptor) propsDesc.get(name);
                if (pd == null) {
                    throw new InvocationBeansException();
                }
                Method method = pd.getWriteMethod();
                if (method == null) {
                    throw new InvocationBeansException();
                }
                if (method != null) {
                    Object value = e.getValue();
                    method.invoke(bean, newParameterValues(new Object[] { value }));
                }
            }
            return bean;

        } catch (ClassCastException e) {
            throw new InvocationBeansException(e);
        } catch (IllegalArgumentException e) {
            throw new InvocationBeansException(e);
        } catch (IllegalAccessException e) {
            throw new InvocationBeansException(e);
        } catch (InvocationTargetException e) {
            throw new InvocationBeansException(e);
        }

    }

    /**
     * ビーンのプロパティを文字列に変換して返却します。
     * 
     * @param bean
     *            ビーンのインスタンス
     * @return ビーンのプロパティの文字列表現
     * @throws InvocationBeansException
     *             プロパティの呼出に失敗した場合
     */
    public static String toBeanString(Object bean) throws InvocationBeansException {

        if (bean == null) {
            throw new NullPointerException("argument is null.");
        }
        try {

            StringBuffer sb = new StringBuffer();
            PropertyDescriptor[] ds = getPropertyDescriptors(bean);

            sb.append('{');
            int count = 0;
            for (int i = 0; i < ds.length; i++) {
                Method method = ds[i].getReadMethod();
                if (method != null) {
                    if (count != 0) {
                        sb.append(',');
                    }
                    Object val = method.invoke(bean, null);
                    sb.append(ds[i].getName());
                    sb.append(':');
                    sb.append(Format.toString(val));
                    count++;
                }
            }
            sb.append('}');

            return sb.toString();

        } catch (IllegalArgumentException e) {
            throw new InvocationBeansException(e);
        } catch (IllegalAccessException e) {
            throw new InvocationBeansException(e);
        } catch (InvocationTargetException e) {
            throw new InvocationBeansException(e);
        }

    }

    /**
     * 指定された参照パターンに基づきプロパティ・マップ・配列・リストの要素に再起的にアクセスして値を返却します。
     * 
     * @param bean
     *            ビーンのインスタンス
     * @param pattern
     *            参照パターン
     * @return 参照パターンの示すネストされたプロパティ
     * @throws InvocationBeansException
     *             プロパティの呼出に失敗した場合
     */
    public static Object getNestedProperty(Object bean, String pattern) throws InvocationBeansException {

        if (Eval.isOrNull(bean, pattern)) {
            throw new NullPointerException("argument is null.");
        }

        final char[] nextChars = "[.".toCharArray();
        try {
            int off = 0;
            Object o = bean;
            while (off < pattern.length()) {
                if (Eval.isCharAt(pattern, off, '[')) {
                    String key = Cutter.cutIndexOf(pattern, ++off, ']');
                    if (o instanceof Map) {
                        o = ((Map) o).get(key);
                    } else {
                        int index = Integer.parseInt(key);
                        if (o instanceof List) {
                            o = ((List) o).get(index);
                        } else if (Eval.isArray(o)) {
                            o = Array.get(o, index);
                        } else {
                            throw new InvocationBeansException("illegal pattern:" + pattern);
                        }
                    }
                    off += key.length() + 1;
                } else {
                    if (Eval.isCharAt(pattern, off, '.')) {
                        off++;
                    } else if (off != 0) {
                        throw new InvocationBeansException("illegal pattern:" + pattern);
                    }

                    int end = Seek.orIndexOf(pattern, nextChars, off);
                    if (Eval.isPositive(end)) {
                        // property 'property. or property['
                        String propertyName = pattern.substring(off, end);
                        o = getProperty(o, propertyName);
                        off = end;
                    } else {// end property 'property'
                        String propertyName = pattern.substring(off);
                        o = getProperty(o, propertyName);
                        off = pattern.length();
                    }
                }
            }
            return o;
        } catch (IndexOutOfBoundsException e) {
            throw new InvocationBeansException("illegal pattern:" + pattern, e);
        } catch (NumberFormatException e) {
            throw new InvocationBeansException("illegal pattern:" + pattern, e);
        }

    }

    /**
     * 指定された参照パターンに基づき引数をCEL式で解析するメソッドとプロパティ・マップ・配列・リストの要素に再起的にアクセスして値を返却します。
     * 
     * @param bean
     *            ビーンのインスタンス
     * @param pattern
     *            参照パターン
     * @return 参照パターンの示すネストされたプロパティ
     * @throws InvocationBeansException
     *             プロパティの呼出に失敗した場合
     */
    public static Object getCELNestedProperty(Object bean, String pattern) throws InvocationBeansException {

        if (Eval.isOrNull(bean, pattern)) {
            throw new NullPointerException("argument is null.");
        }
        try {
            int off = 0;
            Object o = bean;
            CELBinder binder = CELBinder.getObjectCreationBinder();
            while (off < pattern.length()) {
                if (Eval.isCharAt(pattern, off, '[')) {
                    String key = Cutter.cutIndexOf(pattern, ++off, ']');
                    if (o instanceof Map) {
                        o = ((Map) o).get(key);
                    } else {
                        int index = Integer.parseInt(key);
                        if (o instanceof List) {
                            o = ((List) o).get(index);
                        } else if (Eval.isArray(o)) {
                            o = Array.get(o, index);
                        } else {
                            throw new InvocationBeansException("illegal pattern:" + pattern);
                        }
                    }
                    off += key.length() + 1;
                } else {
                    if (Eval.isCharAt(pattern, off, '.')) {
                        off++;
                    } else if (off != 0) {
                        throw new InvocationBeansException("illegal pattern:" + pattern);
                    }
                    int end = Seek.orIndexOf(pattern, "[(.#".toCharArray(), off);
                    if (-1 < end) {
                        if (Eval.isCharAt(pattern, end, '(')) {// method 'method(arg...)'
                            String methodName = pattern.substring(off, end);
                            CELReturnValue args = CELHelper.getArguments(binder, pattern, end, null);
                            Class c = (o instanceof Class) ? (Class) o : o.getClass();
                            o = invokeMethod(c, o, methodName, (Object[]) args.value);
                            off = args.nextIndex;
                        } else if (Eval.isCharAt(pattern, end, '#')) {// field '#field'
                            String fieldName = pattern.substring(off, end);
                            o = getFieldValue(o, fieldName);
                            off = end;
                        } else {// property 'property. or property[num]'
                            String propertyName = pattern.substring(off, end);
                            o = getProperty(o, propertyName);
                            off = end;
                        }
                    } else {
                        String propertyName = pattern.substring(off);
                        o = getProperty(o, propertyName);
                        off = pattern.length();
                    }
                }
            }
            return o;
        } catch (CELFormatException e) {
            throw new InvocationBeansException("illegal pattern:" + pattern, e);
        } catch (IndexOutOfBoundsException e) {
            throw new InvocationBeansException("illegal pattern:" + pattern, e);
        } catch (NumberFormatException e) {
            throw new InvocationBeansException("illegal pattern:" + pattern, e);
        }

    }

    /**
     * ビーンからプロパティ名の示すプロパティを取得して返却します。
     * 
     * @param bean
     *            ビーンのインスタンス
     * @param propertyName
     *            プロパティ名
     * @return プロパティ
     * @throws InvocationBeansException
     *             プロパティの呼出に失敗した場合
     */
    public static Object getProperty(Object bean, String propertyName) throws InvocationBeansException {

        if (Eval.isOrNull(bean, propertyName)) {
            throw new NullPointerException("argument is null.");
        }
        try {

            Class beanClass = bean.getClass();
            Method method = getMatchingAccessibleMethod(beanClass, "get" + capitalize(propertyName));
            if (method == null) {
                method = getMatchingAccessibleMethod(beanClass, "is" + capitalize(propertyName));
            }
            if (method == null) {
                throw new InvocationBeansException("Not Matching property, class:" + beanClass + ", name:"
                        + propertyName);
            }
            return method.invoke(bean, null);

        } catch (IllegalArgumentException e) {
            throw new InvocationBeansException("illegal property:" + propertyName, e);
        } catch (IllegalAccessException e) {
            throw new InvocationBeansException("illegal property:" + propertyName, e);
        } catch (InvocationTargetException e) {
            throw new InvocationBeansException("illegal property:" + propertyName, e);
        }

    }

    /*
     * new instance
     */

    /**
     * パラメータを指定したコンストラクタからインスタンスを生成して返却します。
     * 
     * @param c
     *            クラス
     * @return 生成されたインスタンス
     * @throws InvocationBeansException
     *             インスタンスの生成に失敗した場合
     */
    public static Object newInstance(Class c) throws InvocationBeansException {
        return newInstance(c, new Class[0], new Object[0]);
    }

    /**
     * パラメータを指定したコンストラクタからインスタンスを生成して返却します。
     * 
     * @param c
     *            クラス
     * @param parameterValues
     *            パラメータ値
     * @return 生成されたインスタンス
     * @throws InvocationBeansException
     *             インスタンスの生成に失敗した場合
     */
    public static Object newInstance(Class c, Object[] parameterValues) throws InvocationBeansException {
        return newInstance(c, toClassArray(parameterValues), parameterValues);
    }

    /**
     * パラメータを指定したコンストラクタからインスタンスを生成して返却します。
     * 
     * @param c
     *            クラス
     * @param parameterTypes
     *            パラメータ型
     * @param parameterValues
     *            パラメータ値
     * @return 生成されたインスタンス
     * @throws InvocationBeansException
     *             インスタンスの生成に失敗した場合
     */
    public static Object newInstance(Class c, Class[] parameterTypes, Object[] parameterValues)
            throws InvocationBeansException {

        if (Eval.isOrNull(c, parameterTypes, parameterValues)) {
            throw new NullPointerException("argument is null.");
        }
        try {

            Constructor constructor = getConstructor(c, parameterTypes);
            return constructor.newInstance(newParameterValues(parameterValues));

        } catch (SecurityException e) {
            throw new InvocationBeansException("Class#getConstructor class:" + c, e);
        } catch (IllegalArgumentException e) {
            throw new InvocationBeansException("Constructor#newInstance class:" + c, e);
        } catch (InstantiationException e) {
            throw new InvocationBeansException("Constructor#newInstance class:" + c, e);
        } catch (IllegalAccessException e) {
            throw new InvocationBeansException("Constructor#newInstance class:" + c, e);
        } catch (InvocationTargetException e) {
            throw new InvocationBeansException("Constructor#newInstance class:" + c, e);
        }

    }

    /*
     * invoke method
     */

    /**
     * 指定されたインスタンスとクラスからメソッドを呼び出し結果を返却します。
     * 
     * @param c
     *            実行するクラス
     * @param obj
     *            実行するインスタンス
     * @param methodName
     *            メソッド名
     * @return メソッドの返却値
     * @throws InvocationBeansException
     *             処理の呼出に失敗した場合
     */
    public static Object invokeMethod(Class c, Object obj, String methodName) throws InvocationBeansException {
        return invokeMethod(c, obj, methodName, new Class[0], new Object[0]);
    }

    /**
     * 指定されたインスタンスとクラスからメソッドを呼び出し結果を返却します。
     * 
     * @param c
     *            実行するクラス
     * @param obj
     *            実行するインスタンス
     * @param methodName
     *            メソッド名
     * @param parameterValues
     *            パラメータ値
     * @return メソッドの返却値
     * @throws InvocationBeansException
     *             メソッドの呼出に失敗した場合
     */
    public static Object invokeMethod(Class c, Object obj, String methodName, Object[] parameterValues)
            throws InvocationBeansException {
        return invokeMethod(c, obj, methodName, toClassArray(parameterValues), parameterValues);
    }

    /**
     * 指定されたインスタンスとクラスからメソッドを呼び出し結果を返却します。
     * 
     * @param c
     *            実行するクラス
     * @param obj
     *            実行するインスタンス
     * @param methodName
     *            メソッド名
     * @param parameterTypes
     *            パラメータ型
     * @param parameterValues
     *            パラメータ値
     * @return メソッドの返却値
     * @throws InvocationBeansException
     *             メソッドの呼出に失敗した場合
     */
    public static Object invokeMethod(Class c, Object obj, String methodName, Class[] parameterTypes,
            Object[] parameterValues) throws InvocationBeansException {

        if (Eval.isOrNull(c, methodName, parameterTypes, parameterValues)) {
            throw new NullPointerException("argument is null.");
        }

        Method method = getMatchingAccessibleMethod(c, methodName, parameterTypes);
        if (method == null) {
            throw new InvocationBeansException("Not Matching method, class:" + c + ", name:" + methodName);
        }
        try {

            if (Modifier.isStatic(method.getModifiers())) {
                return method.invoke(null, newParameterValues(parameterValues));
            } else {
                if (obj == null) {
                    throw new NullPointerException("argument obj is null.");
                }
                return method.invoke(obj, newParameterValues(parameterValues));
            }

        } catch (IllegalArgumentException e) {
            throw new InvocationBeansException("Method#invoke class:" + c, e);
        } catch (IllegalAccessException e) {
            throw new InvocationBeansException("Method#invoke class:" + c, e);
        } catch (InvocationTargetException e) {
            throw new InvocationBeansException("Method#invoke class:" + c, e);
        }

    }

    /**
     * 指定されたインスタンスとクラスからメソッドを呼び出し結果を返却します。
     * 
     * @param c
     *            実行するクラス
     * @param obj
     *            実行するインスタンス
     * @param method
     *            メソッド
     * @param parameterValues
     *            パラメータ値
     * @return メソッドの返却値
     * @throws InvocationBeansException
     *             メソッドの呼出に失敗した場合
     */
    public static Object invokeMethod(Class c, Object obj, Method method, Object[] parameterValues)
            throws InvocationBeansException {

        if (Eval.isOrNull(c, method, parameterValues)) {
            throw new NullPointerException("argument is null.");
        }

        try {

            if (Modifier.isStatic(method.getModifiers())) {
                return method.invoke(null, newParameterValues(parameterValues));
            } else {
                if (obj == null) {
                    throw new NullPointerException("argument obj is null.");
                }
                return method.invoke(obj, newParameterValues(parameterValues));
            }

        } catch (IllegalArgumentException e) {
            throw new InvocationBeansException("Method#invoke class:" + c, e);
        } catch (IllegalAccessException e) {
            throw new InvocationBeansException("Method#invoke class:" + c, e);
        } catch (InvocationTargetException e) {
            throw new InvocationBeansException("Method#invoke class:" + c, e);
        }

    }

    /*
     * Field
     */

    /**
     * オブジェクトからフィールド名の示すフィールド値を取得して返却します。
     * 
     * @param obj
     *            フィールドを保有するオブジェクト
     * @param fieldName
     *            フィールド名
     * @return フィールドの値
     * @throws InvocationBeansException
     *             フィールドの呼出に失敗した場合
     */
    public static Object getFieldValue(Object obj, String fieldName) throws InvocationBeansException {
        return getFieldValue(obj.getClass(), obj, fieldName);
    }

    /**
     * オブジェクトからフィールド名の示すフィールド値を取得して返却します。
     * 
     * @param c
     *            フィールドを保有するクラス
     * @param obj
     *            フィールドを保有するオブジェクト
     * @param fieldName
     *            フィールド名
     * @return フィールドの値
     * @throws InvocationBeansException
     *             フィールドの呼出に失敗した場合
     */
    public static Object getFieldValue(Class c, Object obj, String fieldName) throws InvocationBeansException {

        if (Eval.isOrNull(c, fieldName)) {
            throw new NullPointerException("argument is null.");
        }
        try {

            Field field = getField(c, fieldName);
            return field.get(obj);

        } catch (IllegalArgumentException e) {
            throw new InvocationBeansException("illegal field:" + fieldName, e);
        } catch (IllegalAccessException e) {
            throw new InvocationBeansException("illegal field:" + fieldName, e);
        }

    }

    /**
     * オブジェクトからフィールド名の示すフィールド値を設定して元の値を返却します。
     * 
     * @param obj
     *            フィールドを保有するオブジェクト
     * @param fieldName
     *            フィールド名
     * @param newValue
     *            格納するフィールド値
     * @return 元のフィールド値
     * @throws InvocationBeansException
     *             フィールドの呼出に失敗した場合
     */
    public static Object setFieldValue(Object obj, String fieldName, Object newValue) throws InvocationBeansException {
        return setFieldValue(obj.getClass(), obj, fieldName, newValue);
    }

    /**
     * オブジェクトからフィールド名の示すフィールド値を設定して元の値を返却します。
     * 
     * @param c
     *            フィールドを保有するクラス
     * @param obj
     *            フィールドを保有するオブジェクト
     * @param fieldName
     *            フィールド名
     * @param newValue
     *            格納するフィールド値
     * @return 元のフィールド値
     * @throws InvocationBeansException
     *             フィールドの呼出に失敗した場合
     */
    public static Object setFieldValue(Class c, Object obj, String fieldName, Object newValue)
            throws InvocationBeansException {

        if (Eval.isOrNull(c, fieldName)) {
            throw new NullPointerException("argument is null.");
        }
        try {

            Field field = getField(c, fieldName);
            Object oldValue = field.get(obj);
            field.set(obj, newValue);
            return oldValue;

        } catch (IllegalArgumentException e) {
            throw new InvocationBeansException("field#set class:" + c + ", field:" + fieldName + ", value:" + newValue,
                    e);
        } catch (IllegalAccessException e) {
            throw new InvocationBeansException("field#set class:" + c + ", field:" + fieldName + ", value:" + newValue,
                    e);
        }

    }

    /*
     * Accessible
     */

    /**
     * アクセス可能なフィールドを検索して返却します。 <br>
     * フィールドが発見できない場合<code>null</code>を返却します。
     * 
     * @param c
     *            検索するクラス
     * @param fieldName
     *            フィールド名
     * @return アクセス可能なフィールド、発見できない場合<code>null</code>
     */
    public static Field getAccessibleField(Class c, String fieldName) {

        if (Eval.isOrNull(c, fieldName)) {
            throw new NullPointerException("argument is null.");
        }
        Field field = null;
        try {
            field = getAccessibleField(c.getField(fieldName));
        } catch (SecurityException e) {
            // no op
        } catch (NoSuchFieldException e) {
            // no op
        }
        return field;
    }

    /**
     * パラメータが空のアクセス可能なコンストラクタを検索して返却します。 <br>
     * コンストラクタが発見できない場合<code>null</code>を返却します。
     * 
     * @param c
     *            検索するクラス
     * @return アクセス可能なコンストラクタ、発見できない場合<code>null</code>
     */
    public static Constructor getMatchingAccessibleConstructor(Class c) {
        return getMatchingAccessibleConstructor(c, new Class[0]);

    }

    /**
     * パラメータを割り当てられるアクセス可能なコンストラクタを検索して返却します。 <br>
     * コンストラクタが発見できない場合<code>null</code>を返却します。
     * 
     * @param c
     *            検索するクラス
     * @param parameterTypes
     *            引数のパラメータ型
     * @return アクセス可能なコンストラクタ、発見できない場合<code>null</code>
     */
    public static Constructor getMatchingAccessibleConstructor(Class c, Class[] parameterTypes) {

        if (Eval.isOrNull(c, parameterTypes)) {
            throw new NullPointerException("argument is null.");
        }
        try {
            Constructor constructor = getAccessibleConstructor(c.getConstructor(parameterTypes));
            if (constructor != null) {
                return constructor;
            }
        } catch (SecurityException e) {
            // no op
        } catch (NoSuchMethodException e) {
            // no op
        }

        int paramSize = parameterTypes.length;
        Constructor[] constructors = c.getConstructors();
        for (int i = 0, size = constructors.length; i < size; i++) {
            Class[] constructorParams = constructors[i].getParameterTypes();
            int constructorParamSize = constructorParams.length;
            if (constructorParamSize == paramSize) {
                boolean match = true;
                for (int n = 0; n < constructorParamSize; n++) {
                    if (!isAssignmentCompatible(constructorParams[n], parameterTypes[n])) {
                        match = false;
                        break;
                    }
                }
                if (match) {
                    Constructor constructor = getAccessibleConstructor(constructors[i]);
                    if (constructor != null) {
                        return constructor;
                    }
                }
            }
        }
        return null;
    }

    /**
     * 指定されたクラスから処理の対象となるメソッドを返却します。
     * 
     * @param c
     *            クラス
     * @param methodName
     *            メソッド名
     * @param parameterTypes
     *            パラメータ型
     * @return 指定のメソッド
     * @throws NoSuchMethodException
     *             指定のメソッドが発見出来ない場合
     */
    public static Method getMethod(Class c, String methodName, Class[] parameterTypes) throws NoSuchMethodException {

        if (Eval.isOrNull(c, methodName, parameterTypes)) {
            throw new NullPointerException("argument is null.");
        }

        Method method = getMatchingAccessibleMethod(c, methodName, parameterTypes);
        if (method == null) {
            throw new NoSuchMethodException(Format.format("class:{0}, method:{1}, args:{2,array}", c, methodName,
                    parameterTypes));
        }
        return method;

    }

    /**
     * パラメータを持たないアクセス可能なメソッドを検索して返却します。 <br>
     * メソッドが発見できない場合<code>null</code>を返却します。
     * 
     * @param c
     *            検索するクラス
     * @param methodName
     *            検索するメソッド名
     * @return アクセス可能なメソッド、発見できない場合<code>null</code>
     */
    public static Method getMatchingAccessibleMethod(Class c, String methodName) {
        return getMatchingAccessibleMethod(c, methodName, new Class[0]);

    }

    /**
     * パラメータを割り当てられるアクセス可能なメソッドを検索して返却します。 <br>
     * メソッドが発見できない場合<code>null</code>を返却します。
     * 
     * @param c
     *            検索するクラス
     * @param methodName
     *            検索するメソッド名
     * @param parameterType
     *            引数のパラメータ型
     * @return アクセス可能なメソッド、発見できない場合<code>null</code>
     */
    public static Method getMatchingAccessibleMethod(Class c, String methodName, Class parameterType) {
        return getMatchingAccessibleMethod(c, methodName, new Class[] { parameterType });

    }

    /**
     * パラメータを割り当てられるアクセス可能なメソッドを検索して返却します。 <br>
     * メソッドが発見できない場合<code>null</code>を返却します。
     * 
     * @param c
     *            検索するクラス
     * @param methodName
     *            検索するメソッド名
     * @param parameterTypes
     *            引数のパラメータ型
     * @return アクセス可能なメソッド、発見できない場合<code>null</code>
     */
    public static Method getMatchingAccessibleMethod(Class c, String methodName, Class[] parameterTypes) {

        if (Eval.isOrNull(c, methodName, parameterTypes)) {
            throw new NullPointerException("argument is null.");
        }

        try {
            Method method = c.getMethod(methodName, parameterTypes);
            try {
                method.setAccessible(true);
            } catch (SecurityException se) {
                // no op
            }
            return method;
        } catch (SecurityException se) {
            // no op
        } catch (NoSuchMethodException e) {
            // no op
        }

        int paramSize = parameterTypes.length;
        Method[] methods = c.getMethods();
        for (int i = 0, size = methods.length; i < size; i++) {
            if (methods[i].getName().equals(methodName)) {

                Class[] methodsParams = methods[i].getParameterTypes();
                int methodParamSize = methodsParams.length;
                if (methodParamSize == paramSize) {
                    boolean match = true;
                    for (int n = 0; n < methodParamSize; n++) {
                        if (!isAssignmentCompatible(methodsParams[n], parameterTypes[n])) {
                            match = false;
                            break;
                        }
                    }

                    if (match) {
                        Method method = getAccessibleMethod(methods[i]);
                        if (method != null) {
                            try {
                                method.setAccessible(true);
                            } catch (SecurityException se) {
                                // no op
                            }
                            return method;
                        }
                    }
                }
            }
        }
        return null;
    }

    /**
     * パラメータを割り当てられるアクセス可能なプロパティの設定メソッドを検索して返却します。 <br>
     * プロパティの設定メソッドが発見できない場合<code>null</code>を返却します。
     * 
     * @param c
     *            検索するクラス
     * @param propertyName
     *            検索するプロパティ名
     * @param parameterType
     *            引数のプロパティ型
     * @return アクセス可能なプロパティの設定メソッド、発見できない場合<code>null</code>
     */
    public static Method getMatchingAccessibleSetProperty(Class c, String propertyName, Class parameterType) {
        return getMatchingAccessibleSetProperty(c, propertyName, new Class[] { parameterType });
    }

    /**
     * パラメータを割り当てられるアクセス可能なプロパティの設定メソッドを検索して返却します。 <br>
     * プロパティの設定メソッドが発見できない場合<code>null</code>を返却します。
     * 
     * @param c
     *            検索するクラス
     * @param propertyName
     *            検索するプロパティ名
     * @param parameterTypes
     *            引数のプロパティ型
     * @return アクセス可能なプロパティの設定メソッド、発見できない場合<code>null</code>
     */
    public static Method getMatchingAccessibleSetProperty(Class c, String propertyName, Class[] parameterTypes) {
        return getMatchingAccessibleMethod(c, "set" + capitalize(propertyName), parameterTypes);
    }

    /**
     * パラメータを割り当てられるアクセス可能なプロパティの取得メソッドを検索して返却します。 <br>
     * プロパティの取得メソッドが発見できない場合<code>null</code>を返却します。
     * 
     * @param c
     *            検索するクラス
     * @param propertyName
     *            検索するプロパティ名
     * @param parameterType
     *            引数のプロパティ型
     * @return アクセス可能なプロパティの取得メソッド、発見できない場合<code>null</code>
     */
    public static Method getMatchingAccessibleGetProperty(Class c, String propertyName, Class parameterType) {
        return getMatchingAccessibleGetProperty(c, propertyName, new Class[] { parameterType });
    }

    /**
     * パラメータを割り当てられるアクセス可能なプロパティの取得メソッドを検索して返却します。 <br>
     * プロパティの取得メソッドが発見できない場合<code>null</code>を返却します。
     * 
     * @param c
     *            検索するクラス
     * @param propertyName
     *            検索するプロパティ名
     * @param parameterTypes
     *            引数のプロパティ型
     * @return アクセス可能なプロパティの取得メソッド、発見できない場合<code>null</code>
     */
    public static Method getMatchingAccessibleGetProperty(Class c, String propertyName, Class[] parameterTypes) {
        Method method = getMatchingAccessibleMethod(c, "get" + capitalize(propertyName), parameterTypes);
        if (method == null) {
            method = getMatchingAccessibleMethod(c, "is" + capitalize(propertyName), parameterTypes);
        }
        return method;
    }

    /*
     * private
     */

    /* 第一引数に対して第二引数が割り当て可能か検証します。 */
    private static boolean isAssignmentCompatible(Class fromType, Class toType) {
        // 単純に割り当て可能か検証する
        if (fromType.isAssignableFrom(toType)) {
            return true;
        }
        // ラッパー型が一致する場合許可
        if (fromType.isPrimitive()) {
            Class parameterWrapperClass = Boxing.boxClass(fromType);
            if (parameterWrapperClass != null) {
                return parameterWrapperClass.equals(toType);
            }
        }
        return false;
    }

    /* 処理の対象となるコンストラクタを返却します。 */
    private static Constructor getConstructor(Class c, Class[] parameterTypes) throws InvocationBeansException {
        Constructor constructor = getMatchingAccessibleConstructor(c, parameterTypes);
        if (constructor == null) {
            throw new InvocationBeansException(Format.format("class:{0}, args:{1,array}", c, parameterTypes));
        }
        return constructor;
    }

    /* アクセス可能なフィールドを検索して返却します。 */
    private static Field getField(Class c, String fieldName) throws InvocationBeansException {
        Field field = getAccessibleField(c, fieldName);
        if (field == null) {
            throw new InvocationBeansException("Not Matching field, class:" + c + ", name:" + fieldName);
        }
        return field;
    }

    /* フィールド自身と属するクラスまたはクラスが実装するインターフェースが公開されているか検証します。 */
    private static Field getAccessibleField(Field field) {
        if (field == null) {
            return null;
        }
        if (!Modifier.isPublic(field.getModifiers())) {
            return null;
        }
        Class c = field.getDeclaringClass();
        if (Modifier.isPublic(c.getModifiers())) {
            return field;
        }
        field = getAccessibleFieldFromInterfaceNest(c, field.getName());
        return field;
    }

    /* フィールドのクラスが実装するインターフェースが公開されているか再起的に検証します。 */
    private static Field getAccessibleFieldFromInterfaceNest(Class c, String fieldName) {

        Field field = null;
        for (; c != null; c = c.getSuperclass()) {

            Class[] interfaces = c.getInterfaces();
            for (int i = 0; i < interfaces.length; i++) {

                if (!Modifier.isPublic(interfaces[i].getModifiers())) {
                    continue;
                }

                try {
                    field = interfaces[i].getDeclaredField(fieldName);
                } catch (NoSuchFieldException e) {
                    // no op
                } catch (SecurityException e) {
                    // no op
                }
                if (field != null) {
                    break;
                }

                field = getAccessibleFieldFromInterfaceNest(interfaces[i], fieldName);
                if (field != null) {
                    break;
                }
            }
        }
        if (field != null) {
            return field;
        }
        return null;

    }

    /* メソッド自身と属するクラスまたはクラスが実装するインターフェースが公開されているか検証します。 */
    private static Method getAccessibleMethod(Method method) {
        if (method == null) {
            return null;
        }
        if (!Modifier.isPublic(method.getModifiers())) {
            return null;
        }
        Class c = method.getDeclaringClass();
        if (Modifier.isPublic(c.getModifiers())) {
            return method;
        }
        method = getAccessibleMethodFromInterfaceNest(c, method.getName(), method.getParameterTypes());
        return method;
    }

    /* コンストラクタと属するクラスが公開されているか検証します。 */
    private static Constructor getAccessibleConstructor(Constructor constructor) {
        if (constructor == null) {
            return null;
        }
        if (!Modifier.isPublic(constructor.getModifiers())) {
            return null;
        }
        Class c = constructor.getDeclaringClass();
        if (Modifier.isPublic(c.getModifiers())) {
            return constructor;
        }
        return null;
    }

    /* メソッドのクラスが実装するインターフェースが公開されているか再起的に検証します。 */
    private static Method getAccessibleMethodFromInterfaceNest(Class c, String methodName, Class[] parameterTypes) {

        Method method = null;
        for (; c != null; c = c.getSuperclass()) {

            Class[] interfaces = c.getInterfaces();
            for (int i = 0; i < interfaces.length; i++) {

                if (!Modifier.isPublic(interfaces[i].getModifiers())) {
                    continue;
                }

                try {
                    method = interfaces[i].getDeclaredMethod(methodName, parameterTypes);
                } catch (NoSuchMethodException e) {
                    // no op
                } catch (SecurityException e) {
                    // no op
                }
                if (method != null) {
                    break;
                }

                method = getAccessibleMethodFromInterfaceNest(interfaces[i], methodName, parameterTypes);
                if (method != null) {
                    break;
                }
            }
        }
        if (method != null) {
            return method;
        }
        return null;
    }

    /*
     * helper
     */

    /* PropertyDescriptorをプロパティ名をキーとしてマップに格納して返却します。 */
    private static Map getMapPropertyDescriptors(Object bean) {
        HashMap propsDesc = new HashMap();
        PropertyDescriptor[] ds = getPropertyDescriptors(bean);
        for (int i = 0; i < ds.length; i++) {
            propsDesc.put(ds[i].getName(), ds[i]);
        }
        return propsDesc;
    }

    /* オブジェクトの配列を基にその要素のクラスの配列を生成して返却する。 */
    private static Class[] toClassArray(Object[] os) {
        Class[] cs = new Class[os.length];
        for (int i = 0; i < os.length; i++) {
            cs[i] = os[i].getClass();
        }
        return cs;
    }

    /* 新規のパラメータの値を生成して返却します。 */
    private static Object[] newParameterValues(Object[] srcValues) throws InvocationBeansException {
        if (srcValues == null) {
            throw new NullPointerException("argument is null.");
        }
        Object[] newValues;
        try {
            newValues = new Object[srcValues.length];
            for (int i = 0; i < srcValues.length; i++) {
                Object o = srcValues[i];
                if (o instanceof ObjectGenerator) {
                    o = ((ObjectGenerator) o).getInstance();
                }
                newValues[i] = o;
            }
        } catch (ObjectCreationException e) {
            if (e.getCause() instanceof InvocationBeansException) {
                throw (InvocationBeansException) e.getCause();
            }
            throw new InvocationBeansException(e);
        }
        return newValues;
    }

    /* 先頭文字を大文字に変換します。 */
    private static String capitalize(String s) {
        if (s == null || s.length() == 0) {
            return s;
        }
        char[] chars = s.toCharArray();
        chars[0] = Character.toUpperCase(chars[0]);
        return new String(chars);
    }
}
