/*
 * shohaku Copyright (C) 2005 tomoya nagatani
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */
package shohaku.core.beans;

/**
 * オブジェクトのフィールド情報を格納する機能を提供します。
 */
public class FieldInfo extends ClassFeatureInfo {

    /* フィールドの名前。 */
    private String name;

    /**
     * デフォルトコンストラクタ。
     */
    public FieldInfo() {
        // no op
    }

    /**
     * 引数のプロパティを格納して初期化します。
     * 
     * @param name
     *            フィールドの名前
     */
    public FieldInfo(String name) {
        setName(name);
    }

    /**
     * 引数のプロパティを格納して初期化します。
     * 
     * @param name
     *            フィールドの名前
     * @param objectType
     *            オブジェクト型
     */
    public FieldInfo(String name, Class objectType) {
        setName(name);
        setObjectType(objectType);
    }

    /**
     * 登録されているクラスからフィールドの値を返却します。
     * 
     * @return フィールドの値
     * @throws InvocationBeansException
     *             処理の呼出に失敗した場合発生する
     */
    public Object get() throws InvocationBeansException {
        return getValue(getObjectType(), null);
    }

    /**
     * 指定されたクラスからフィールドの値を返却します。
     * 
     * @param c
     *            クラス
     * @return フィールドの値
     * @throws InvocationBeansException
     *             処理の呼出に失敗した場合発生する
     */
    public Object get(Class c) throws InvocationBeansException {
        return getValue(c, null);
    }

    /**
     * 指定されたインスタンスからフィールドの値を返却します。
     * 
     * @param obj
     *            インスタンス
     * @return フィールドの値
     * @throws InvocationBeansException
     *             処理の呼出に失敗した場合発生する
     */
    public Object get(Object obj) throws InvocationBeansException {
        if (obj == null) {
            throw new NullPointerException("argument is null.");
        }
        return getValue(obj.getClass(), obj);
    }

    /**
     * 登録されているクラスのフィールドの値を変更し以前の値を返却します。
     * 
     * @param newValue
     *            格納するフィールドの値
     * @return 格納前のフィールドの値
     * @throws InvocationBeansException
     *             処理の呼出に失敗した場合発生する
     */
    public Object set(Object newValue) throws InvocationBeansException {
        return setValue(getObjectType(), null, newValue);
    }

    /**
     * 指定されたクラスのフィールドの値を変更し以前の値を返却します。
     * 
     * @param c
     *            クラス
     * @param newValue
     *            格納するフィールドの値
     * @return 格納前のフィールドの値
     * @throws InvocationBeansException
     *             処理の呼出に失敗した場合発生する
     */
    public Object set(Class c, Object newValue) throws InvocationBeansException {
        return setValue(c, null, newValue);
    }

    /**
     * 指定されたインスタンスのフィールドの値を変更し以前の値を返却します。
     * 
     * @param obj
     *            インスタンス
     * @param newValue
     *            格納するフィールドの値
     * @return 格納前のフィールドの値
     * @throws InvocationBeansException
     *             処理の呼出に失敗した場合発生する
     */
    public Object set(Object obj, Object newValue) throws InvocationBeansException {
        if (obj == null) {
            throw new NullPointerException("argument obj is null.");
        }
        return setValue(obj.getClass(), obj, newValue);
    }

    /*
     * private
     */

    /* 指定されたクラスとそのインスタンスからフィールドの値を返却します。 */
    private Object getValue(Class c, Object obj) throws InvocationBeansException {
        return BeanUtilities.getFieldValue(c, obj, getName());
    }

    /* 指定されたクラスとそのインスタンスのフィールドの値を変更し以前の値を返却します。 */
    private Object setValue(Class c, Object obj, Object newValue) throws InvocationBeansException {
        return BeanUtilities.setFieldValue(c, obj, getName(), newValue);

    }

    /*
     * Property
     */

    /**
     * フィールドの名前を返却します。
     * 
     * @return フィールドの名前
     */
    public String getName() {
        return name;
    }

    /**
     * フィールドの名前を格納します。 <br>
     * 
     * @param name
     *            フィールドの名前
     */
    public void setName(String name) {
        if (name == null) {
            throw new NullPointerException("argument is null.");
        }
        this.name = name;
    }

}
