/*
 * shohaku
 * Copyright (C) 2005  tomoya nagatani
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package shohaku.core.collections.params;

import shohaku.core.collections.KayValueIterator;
import shohaku.core.lang.IllegalIdentifierException;

/**
 * 媒介変数のマッピングを格納する、要素の値を変更可能とするデータ構造を提供します。 <br>
 * 要素の値を変更可能とする AbstractParameters の拡張として定義されています。
 */
abstract class AbstractModifiableParameters extends AbstractParameters {

    /**
     * 更新メソッドを持つ引数エントリを提供します。
     */
    abstract static class AbstractModifiableEntry extends AbstractEntry {

        /**
         * エントリ名とエントリ値を設定して初期化します。
         * 
         * @param name
         *            エントリ名
         * @param value
         *            エントリ値
         * @param hashCode
         *            エントリのハッシュ値
         * @param index
         *            エントリのインデックス
         */
        AbstractModifiableEntry(String name, Object value, int hashCode, int index) {
            super(name, value, hashCode, index);
        }

        /**
         * エントリの値を格納します。 <br>
         * 既存の値が存在する場合その値を返し、存在しない場合<code>null</code>を返却します。
         * 
         * @param value
         *            新たなエントリの値
         * @return 既存の値が存在する場合その値、存在しない場合<code>null</code>
         */
        public Object setValue(Object value) {
            Object oldValue = value;
            this.value = value;
            return oldValue;
        }
    }

    /**
     * 空の引数リストを初期化します。
     */
    AbstractModifiableParameters() {
        super();
    }

    /**
     * 引数の識別子と値の反復子と同エントリを持つ引数リストを初期化します。
     * 
     * @param i
     *            基となる識別子と値の反復子
     */
    AbstractModifiableParameters(KayValueIterator i) {
        super(i);
    }

    /**
     * 引数リストの値の変更不可能かつ削除不可能な反復子を生成して返却します。
     * 
     * @return 引数リストの値の変更不可能かつ削除不可能な反復子
     */
    public KayValueIterator kayValueIterator() {
        return new ModifiableParametersKayValueIterator();
    }

    /**
     * 引数リストの反復子。
     */
    class ModifiableParametersKayValueIterator extends ParametersKayValueIterator {

        /* エントリの値を格納します。 */
        public Object setValue(Object value) {
            check();
            return ((AbstractModifiableEntry) e).setValue(value);
        }
    }

    /**
     * 引数名の引数値を格納し、既存の引数値を返却します。 <br>
     * 指定された名前が存在しない場合は<code>IllegalIdentifierException</code>が発生します。
     * 
     * @param name
     *            引数名
     * @param value
     *            新たな引数値
     * @return 既存の引数値
     * @throws IllegalIdentifierException
     *             引数名が存在しない場合
     */
    public Object setValue(String name, Object value) {
        AbstractModifiableEntry e = (AbstractModifiableEntry) getEntry(name);
        if (null != e) {
            return swapValue(e, value);
        }
        throw new IllegalIdentifierException("name:" + name);
    }

    /**
     * インデックスの引数値を格納し、既存の引数値を返却します。 <br>
     * 指定されたインデックスが存在しない場合は<code>IllegalIdentifierException</code>が発生します。
     * 
     * @param index
     *            インデックス
     * @param value
     *            新たな引数値
     * @return 既存の引数値
     * @throws IllegalIdentifierException
     *             インデックスが存在しない場合
     */
    public Object setValue(int index, Object value) {
        AbstractModifiableEntry e = (AbstractModifiableEntry) getEntry(index);
        if (null != e) {
            return swapValue(e, value);
        }
        throw new IllegalIdentifierException("index:" + index);
    }

    /**
     * エントリ値を入れ替えて古い値を返却します。
     * 
     * @param entry
     *            基のエントリ
     * @param value
     *            先のエントリ
     * @return 古い値
     */
    static Object swapValue(AbstractModifiableEntry entry, Object value) {
        Object old = entry.getValue();
        entry.setValue(value);
        return old;
    }
}