/*
 * shohaku
 * Copyright (C) 2005  tomoya nagatani
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package shohaku.core.io;

import java.nio.charset.Charset;
import java.nio.charset.CharsetEncoder;
import java.nio.charset.IllegalCharsetNameException;
import java.nio.charset.UnsupportedCharsetException;

/**
 * 文字セットのユーティリティメソッドを提供します。
 */
public class CharsetUtils {

    /*
     * Encode
     */

    /**
     * 指定の文字セットで指定された文字をエンコードできるかを評価します。
     * 
     * @param c
     *            評価する文字
     * @param charsetName
     *            要求された文字セットの名前 (標準名または別名)
     * @return 指定の文字セットで指定された文字をエンコードできる場合にかぎり<code>true</code>
     * @throws UnsupportedCharsetException
     *             指定された文字セット名が不当である場合
     * @throws IllegalCharsetNameException
     *             指定された文字セットを現在の Java 仮想マシンでは利用できない場合
     * @throws UnsupportedOperationException
     *             この文字セットがエンコードをサポートしない場合
     */
    public static boolean isEncode(char c, String charsetName) throws UnsupportedCharsetException,
            IllegalCharsetNameException, UnsupportedOperationException {

        return isEncode(c, Charset.forName(charsetName));

    }

    /**
     * 指定の文字セットで指定された文字をエンコードできるかを評価します。
     * 
     * @param c
     *            評価する文字
     * @param charset
     *            文字セット
     * @return 指定の文字セットで指定された文字をエンコードできる場合にかぎり<code>true</code>
     * @throws UnsupportedOperationException
     *             この文字セットがエンコードをサポートしない場合
     */
    public static boolean isEncode(char c, Charset charset) throws UnsupportedOperationException {
        CharsetEncoder cEncoder = charset.newEncoder();
        return cEncoder.canEncode(c);
    }

    /**
     * 指定の文字セットで指定された文字シーケンスをエンコードできるかを評価します。
     * 
     * @param cs
     *            評価する文字シーケンス
     * @param charsetName
     *            要求された文字セットの名前 (標準名または別名)
     * @return 指定の文字セットで指定された文字をエンコードできる場合にかぎり<code>true</code>
     * @throws UnsupportedCharsetException
     *             指定された文字セット名が不当である場合
     * @throws IllegalCharsetNameException
     *             指定された文字セットを現在の Java 仮想マシンでは利用できない場合
     * @throws UnsupportedOperationException
     *             この文字セットがエンコードをサポートしない場合
     */
    public static boolean isEncode(CharSequence cs, String charsetName) throws UnsupportedCharsetException,
            IllegalCharsetNameException, UnsupportedOperationException {

        return isEncode(cs, Charset.forName(charsetName));

    }

    /**
     * 指定の文字セットで指定された文字シーケンスをエンコードできるかを評価します。
     * 
     * @param cs
     *            評価する文字シーケンス
     * @param charset
     *            文字セット
     * @return 指定の文字セットで指定された文字をエンコードできる場合にかぎり<code>true</code>
     * @throws UnsupportedOperationException
     *             この文字セットがエンコードをサポートしない場合
     */
    public static boolean isEncode(CharSequence cs, Charset charset) throws UnsupportedOperationException {
        CharsetEncoder cEncoder = charset.newEncoder();
        return cEncoder.canEncode(cs);
    }

}
