/*
 * shohaku
 * Copyright (C) 2005  tomoya nagatani
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package shohaku.core.text;

import java.sql.Timestamp;
import java.util.Calendar;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;

import shohaku.core.collections.IteratorUtils;
import shohaku.core.lang.VariableInteger;
import shohaku.core.util.DateUtils;

/**
 * オブジェクトとその状態に関する記述文を生成するユーティリティメソッドを提供します。
 */
class ToStringHelper {

    /** 値を出力する処理数の上限。 */
    private static final int REPETITION_LIMIT = 1024;

    /* 通常の出力。 */
    static final int NORMAL = 0;

    /* クラス名の出力。 */
    static final int CLASSES = 1;

    /* 深層内容のクラス名の出力。 */
    static final int DEEP_CLASSES = 2;

    /*
     * toString
     */

    /**
     * 指定のスタイルでオブジェクトのより詳細な文字列表現を構築します。 <br>
     * 指定されたオブジェクトが配列の場合は配列の内容の文字列表現を返却します。 <br>
     * 配列または Map, Collection の要素が配列の場合にその要素の内容の文字列表現を含んだ文字列を返却します。 <br>
     * 以外の場合は<code>String.valueOf(o)</code>により文字列に変換されます。 <br>
     * <br>
     * o が配列の場合は、角括弧 (「[]」) で囲まれた配列要素のリストで構成されます。 <br>
     * 隣接する要素は、文字「, 」(コンマおよび空白文字) で区切られます。 <br>
     * 要素はその型に対応する<code>String.valueOf(#type# o)</code>により文字列に変換されます。 <br>
     * <br>
     * o が<code> null </code>の場合<code> 「null」 </code>を返却します。
     * 
     * @param o
     *            文字列表現を返す配列
     * @param style
     *            書式スタイル
     * @return o の文字列表現
     */
    static String toString(Object o, int style) {
        return toString(o, style, new VariableInteger());
    }

    /* 指定された配列の内容の文字列表現を返却します。 */
    private static String toString(Object o, int style, VariableInteger count) {
        if (null == o) {
            return "null";
        }
        StringBuffer buf = new StringBuffer();
        appendObject(o, buf, style, count);
        return buf.toString();
    }

    /* 指定された配列の内容の文字列表現を返却します。 */
    private static String toString(Iterator i, int style, VariableInteger count) {
        StringBuffer buf = new StringBuffer();
        buf.append('[');

        boolean hasNext = i.hasNext();
        while (hasNext) {
            if (appendObject(i.next(), buf, style, count)) {
                break;
            }
            hasNext = i.hasNext();
            if (hasNext) {
                buf.append(", ");
            }
        }

        buf.append(']');
        return buf.toString();
    }

    /* 指定されたマップの内容の文字列表現を返却します。 */
    private static String toString(Map m, int style, VariableInteger count) {
        if (m == null) {
            return "null";
        }
        StringBuffer buf = new StringBuffer();
        if (m.size() == 0) {
            buf.append("{}");
        } else {
            buf.append('{');
            Iterator i = m.entrySet().iterator();
            boolean hasNext = i.hasNext();
            while (hasNext) {
                Map.Entry e = (Map.Entry) i.next();
                Object key = e.getKey();
                Object value = e.getValue();
                if (key == m) {
                    buf.append("(this Map)");
                } else {
                    if (appendObject(key, buf, style, count)) {
                        break;
                    }
                }
                buf.append(':');
                if (value == m) {
                    buf.append("(this Map)");
                } else {
                    if (appendObject(value, buf, style, count)) {
                        break;
                    }
                }
                hasNext = i.hasNext();
                if (hasNext) {
                    buf.append(", ");
                }
            }
            buf.append('}');
        }
        return buf.toString();
    }

    /* 指定されたコレクションの内容の文字列表現を返却します。 */
    private static String toString(Collection c, int style, VariableInteger count) {
        if (c == null) {
            return "null";
        }
        StringBuffer buf = new StringBuffer();
        if (c.size() == 0) {
            buf.append("[]");
        } else {
            buf.append('[');
            Iterator i = c.iterator();
            boolean hasNext = i.hasNext();
            while (hasNext) {
                Object e = i.next();
                if (e == c) {
                    buf.append("(this Collection)");
                } else {
                    if (appendObject(e, buf, style, count)) {
                        break;
                    }
                }
                hasNext = i.hasNext();
                if (hasNext) {
                    buf.append(", ");
                }
            }
            buf.append(']');
        }
        return buf.toString();
    }

    /* 値のクラス情報を追加します。 */
    private static void appendClass(Object o, StringBuffer buf) {
        buf.append('<');
        buf.append(o.getClass().getName());
        buf.append('>');
    }

    /* 値を追加します。 */
    private static boolean appendObject(Object o, StringBuffer buf, int style, VariableInteger count) {
        if (count.intValue() >= REPETITION_LIMIT) {
            buf.append("...");
            return true;
        }
        if (DEEP_CLASSES == style) {
            appendClass(o, buf);
        } else if (CLASSES == style && count.intValue() == 0) {
            appendClass(o, buf);
        }
        count.increment();
        if (o == null) {
            buf.append("null");
            return false;
        }
        if (o.getClass().isArray()) {
            if (o instanceof Object[]) {
                // Object[]
                buf.append(toString(IteratorUtils.asIterator((Object[]) o), style, count));
            } else {
                // Primitive
                buf.append(toString(IteratorUtils.toStringArrayIterator(o), style, count));
            }
        } else if (o instanceof Map) {
            buf.append(toString((Map) o, style, count));
        } else if (o instanceof Collection) {
            buf.append(toString((Collection) o, style, count));
        } else if (o instanceof Timestamp) {
            buf.append(String.valueOf(o));
        } else if (o instanceof java.sql.Time) {
            buf.append(String.valueOf(o));
        } else if (o instanceof java.util.Date) {
            buf.append(DateUtils.format(o));
        } else if (o instanceof Calendar) {
            buf.append(DateUtils.format(o));
        } else {
            buf.append(String.valueOf(o));
        }
        return false;
    }

}
