/*
 * shohaku Copyright (C) 2005 tomoya nagatani
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */
package shohaku.core.util;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.charset.Charset;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import shohaku.core.util.codec.PropertiesEscapesCoder;

/**
 * 拡張プロパティリストの抽象実装を提供します。 <br>
 * 多様な実装が想定されるため、この抽象実装ではシリアライズ機能を定義しません。
 */
public abstract class AbstractXProperties {

    /** デフォルトで使用する文字セット(ISO 8859-1). */
    protected static final Charset DEFAULT_CHARSET = Charset.forName("8859_1");

    /** キーと値の以外の文字。 */
    protected static final String keyValueSeparators = "=: \t\r\n\f";

    /** キーと値の区切り文字。 */
    protected static final String strictKeyValueSeparators = "=:";

    /** コメント文を示す文字。 */
    protected static final String commentChars = "#!";

    /** スペースとして扱う文字。 */
    protected static final String whiteSpaceChars = " \t\r\n\f";

    /** プロパティを同期Mapで保管します。 */
    protected final Map lookup = createLookupMap();

    /** プロパティキーを普遍Setで保管します。 */
    protected final Set unmodifiableKeys = Collections.unmodifiableSet(lookup.keySet());

    /**
     * 入力ストリームからキーと要素が対になったプロパティを読み込みます。 <br>
     * ストリームは、引数<code>charset</code>で指定された文字エンコーディングを使用しているとみなされます。
     * <P>
     * 
     * @param inStream
     *            入力ストリーム
     * @param charset
     *            文字エンコーディング
     * @param isUnicodeEscapes
     *            Unicode escapes フラグ
     * @throws IOException
     *             入力ストリームからの読み込み中にエラーが発生した場合
     */
    protected synchronized void loadImpl(InputStream inStream, Charset charset, boolean isUnicodeEscapes)
            throws IOException {

        BufferedReader in = new BufferedReader(new InputStreamReader(inStream, charset));
        while (true) {
            String line = in.readLine();
            if (line == null)
                return;

            if (line.length() > 0) {

                int len = line.length();
                int keyStart;
                for (keyStart = 0; keyStart < len; keyStart++) {
                    if (!isWhiteSpaceChars(line.charAt(keyStart))) {
                        break;
                    }
                }
                if (keyStart == len) {
                    continue;
                }
                char firstChar = line.charAt(keyStart);
                if (isNotCommentChars(firstChar)) {
                    while (continueLine(line)) {
                        String nextLine = in.readLine();
                        if (nextLine == null) {
                            nextLine = "";
                        }
                        String loppedLine = line.substring(0, len - 1);
                        int startIndex;
                        for (startIndex = 0; startIndex < nextLine.length(); startIndex++) {
                            if (!isWhiteSpaceChars(nextLine.charAt(startIndex))) {
                                break;
                            }
                        }
                        nextLine = nextLine.substring(startIndex, nextLine.length());
                        line = new String(loppedLine + nextLine);
                        len = line.length();
                    }

                    int separatorIndex;
                    for (separatorIndex = keyStart; separatorIndex < len; separatorIndex++) {
                        char currentChar = line.charAt(separatorIndex);
                        if (currentChar == '\\') {
                            separatorIndex++;
                        } else if (isKeyValueSeparators(currentChar)) {
                            break;
                        }
                    }

                    int valueIndex;
                    for (valueIndex = separatorIndex; valueIndex < len; valueIndex++) {
                        if (!isWhiteSpaceChars(line.charAt(valueIndex))) {
                            break;
                        }
                    }
                    if (valueIndex < len) {
                        if (isKeyValueSeparators(line.charAt(valueIndex))) {
                            valueIndex++;
                        }
                    }
                    while (valueIndex < len) {
                        if (!isWhiteSpaceChars(line.charAt(valueIndex))) {
                            break;
                        }
                        valueIndex++;
                    }
                    String key = line.substring(keyStart, separatorIndex);
                    String value = (separatorIndex < len) ? line.substring(valueIndex, len) : "";

                    if (isUnicodeEscapes) {
                        key = PropertiesEscapesCoder.decode(key);
                        value = PropertiesEscapesCoder.decode(value);
                    }
                    putProperty(key, value);
                }
            }
        }
    }

    /**
     * 次行を現在の行の継続として扱う場合は<code>true</code>を返却します。 <br>
     * (行末が '\' である場合は次行を現在の行の継続として扱う)
     * 
     * @param line
     *            検証する行文字列
     * @return 継続として扱う場合<code>true</code>
     */
    protected boolean continueLine(String line) {
        int slashCount = 0;
        int index = line.length() - 1;
        while ((index >= 0) && (line.charAt(index--) == '\\')) {
            slashCount++;
        }
        return (slashCount % 2 == 1);
    }

    /**
     * コメント列を示す文字の以外の場合は<code>true</code>を返却します。
     * 
     * @param c
     *            検証する文字
     * @return コメント列を示す文字の以外の場合<code>true</code>
     */
    protected boolean isNotCommentChars(char c) {
        return (commentChars.indexOf(c) == -1);
    }

    /**
     * キーと値の区切り文字の場合<code>true</code>を返却します。
     * 
     * @param c
     *            検証すう文字
     * @return キーと値の区切り文字の場合<code>true</code>
     * @see shohaku.core.util.AbstractXProperties#isKeyValueSeparators(char)
     */
    protected boolean isKeyValueSeparators(char c) {
        return (keyValueSeparators.indexOf(c) != -1);
    }

    /**
     * スペース文字の場合<code>true</code>を返却します。
     * 
     * @param c
     *            検証すう文字
     * @return スペース文字の場合<code>true</code>
     * @see shohaku.core.util.AbstractXProperties#isWhiteSpaceChars(char)
     */
    protected boolean isWhiteSpaceChars(char c) {
        return (whiteSpaceChars.indexOf(c) != -1);
    }

    /**
     * プロパティリストを格納するマップを返却します。
     * 
     * @return プロパティリストを格納するマップ
     */
    protected Map createLookupMap() {
        return new HashMap();
    }

    /**
     * このプロパティにあるすべてのキーの反復子を返却します。
     * 
     * @return プロパティにあるすべてのキーの反復子
     */
    public Iterator keyIterator() {
        return unmodifiableKeys.iterator();
    }

    /**
     * プロパティリストをマップにコピーして返却します。
     * 
     * @return プロパティリストをコピーしたマップ
     */
    public Map toMap() {
        return new HashMap(lookup);
    }

    /*
     * get Property
     */

    /**
     * 指定されたキーを持つプロパティを、プロパティから探します。 <br>
     * プロパティキーがない場合は<code>null</code>が返されます。
     * 
     * @param key
     *            プロパティキー
     * @return プロパティキーが示す値
     */
    public Object getProperty(Object key) {
        return lookup.get(key);
    }

    /**
     * 指定されたキーを持つプロパティを、プロパティから探します。 <br>
     * プロパティキーがない場合は、デフォルト値の引数が返されます。
     * 
     * @param key
     *            プロパティキー
     * @param defaultValue
     *            デフォルト値
     * @return プロパティキーが示す値又は<code>defaultValue</code>
     */
    public Object getProperty(Object key, Object defaultValue) {
        Object o = getProperty(key);
        return (o != null) ? o : defaultValue;
    }

    /**
     * 指定されたキーがプロパティに含まれている場合に<code>true</code>を返却します。
     * 
     * @param key
     *            プロパティキー
     * @return 指定されたキーが含まれている場合<code>true</code>
     */
    public boolean containsKey(Object key) {
        return lookup.containsKey(key);
    }

    /**
     * プロパティの文字列表現を返却します。
     * 
     * @return プロパティの文字列表現
     * @see java.lang.Object#toString()
     */
    public String toString() {
        StringBuffer buf = new StringBuffer();
        buf.append("{");

        Iterator i = lookup.entrySet().iterator();
        boolean hasNext = i.hasNext();
        while (hasNext) {
            Map.Entry e = (Map.Entry) (i.next());
            Object key = e.getKey();
            Object value = e.getValue();
            buf.append(key);
            buf.append('=');
            buf.append(value);
            hasNext = i.hasNext();
            if (hasNext) {
                buf.append(", ");
            }
        }

        buf.append("}");
        return buf.toString();
    }

    /*
     * abstract
     */

    /**
     * プロパティを登録します。
     * 
     * @param key
     *            プロパティキー
     * @param value
     *            プロパティ値
     * @throws IOException
     *             プロパティの登録中にエラーが発生した場合
     */
    abstract protected void putProperty(String key, String value) throws IOException;

    /**
     * 入力ストリームからキーと要素が対になったプロパティリストを読み込みます。 <br>
     * ストリームはデフォルトの<code>ISO 8859-1</code>文字エンコーディングを使用しているとみなされます。 <br>
     * このエンコーディングに直接表示できない文字には<code>Unicode escapes</code>が使用されます。 <br>
     * ただし、エスケープシーケンスでは 1 文字の「u」だけが使用可能です。 <br>
     * 他の文字エンコーディングとプロパティファイルを変換する場合<code>native2ascii</code>ツールを使用できます。
     * 
     * @param inStream
     *            入力ストリーム
     * @throws IOException
     *             入力ストリームからの読み込み中にエラーが発生した場合
     */
    abstract public void load(InputStream inStream) throws IOException;

    /**
     * 入力ストリームからキーと要素が対になったプロパティリストを読み込みます。 <br>
     * ストリームは、引数 charsetName で指定された文字エンコーディングを使用しているとみなされます。 <br>
     * このメソッドは以下と等価です。 <br>
     * <code>load(in, Charset.forName(charsetName));</code>
     * 
     * @param inStream
     *            入力ストリーム
     * @param charsetName
     *            文字エンコーディング
     * @throws IOException
     *             入力ストリームからの読み込み中にエラーが発生した場合
     */
    abstract public void load(InputStream inStream, String charsetName) throws IOException;

    /**
     * 入力ストリームからキーと要素が対になったプロパティリストを読み込みます。 <br>
     * ストリームは、引数 charset で指定された文字エンコーディングを使用しているとみなされます。
     * 
     * @param inStream
     *            入力ストリーム
     * @param charset
     *            文字エンコーディング
     * @throws IOException
     *             入力ストリームからの読み込み中にエラーが発生した場合
     */
    abstract public void load(InputStream inStream, Charset charset) throws IOException;

}
