/*
 * shohaku
 * Copyright (C) 2005  tomoya nagatani
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package shohaku.core.util;

import java.util.Locale;
import java.util.Map;
import java.util.MissingResourceException;

/**
 * ロケールを階層データとする、{@link shohaku.core.util.CELProperties} を用いた階層化リソースバンドルを提供します。
 */
public class CELResourceBundle extends XResourceBundle {

    /* リソースバンドルのキャッシュ機能のシングルトンインスタンスを格納します。 */
    private static final class CELResourceBundleCacheInstance {

        // new Singleton Instance
        static final XResourceBundleCache instance = new XResourceBundleCache(new CELResourceBundleCreater());

    }

    /**
     * プロパティリストを初期化します。
     * 
     * @param parent
     *            親バンドル
     * @param bundleBase
     *            束縛基準
     * @param resources
     *            リソース
     */
    CELResourceBundle(XResourceBundle parent, Object bundleBase, Map resources) {
        super(parent, bundleBase, resources);
    }

    /**
     * ロケールを取得して返却します。
     * 
     * @return ロケール
     */
    public Locale getLocale() {
        return (Locale) getBundleBase();
    }

    /*
     * static
     */

    /**
     * 指定された基底名、デフォルトのロケール、および呼び出し側のクラスローダを使用して、リソースバンドルを取得します。 <br>
     * このメソッドを呼び出すことは、以下を呼び出すことと同じです。 <br>
     * getBundle(baseName, Locale.getDefault(), CELResourceBundle.class.getClassLoader())
     * 
     * @param baseName
     *            基底名
     * @return 指定された基底名とデフォルトのロケールのリソースバンドル
     * @throws MissingResourceException
     *             指定された基底名のリソースバンドルが見つからない場合
     */
    public static CELResourceBundle getBundle(String baseName) {
        return getBundle(baseName, Locale.getDefault(), getDefaultLoader());
    }

    /**
     * 指定された基底名、クラスローダ、およびデフォルトのロケールを使用して、リソースバンドルを取得します。 <br>
     * このメソッドを呼び出すことは、以下を呼び出すことと同じです。 <br>
     * getBundle(baseName, Locale.getDefault(), loader)
     * 
     * @param baseName
     *            基底名
     * @param loader
     *            リソースのロード元のクラスローダ
     * @return 指定された基底名とロケールのリソースバンドル
     * @throws MissingResourceException
     *             指定された基底名のリソースバンドルが見つからない場合
     */
    public static CELResourceBundle getBundle(String baseName, ClassLoader loader) {
        return getBundle(baseName, Locale.getDefault(), loader);
    }

    /**
     * 指定された基底名、ロケール、および呼び出し側のクラスローダを使用して、リソースバンドルを取得します。 <br>
     * このメソッドを呼び出すことは、以下を呼び出すことと同じです。 <br>
     * getBundle(baseName, locale, CELResourceBundle.class.getClassLoader())
     * 
     * @param baseName
     *            基底名
     * @param locale
     *            ロケール
     * @return 指定された基底名とロケールのリソースバンドル
     * @throws MissingResourceException
     *             指定された基底名のリソースバンドルが見つからない場合
     */
    public static CELResourceBundle getBundle(String baseName, Locale locale) {
        return getBundle(baseName, locale, getDefaultLoader());
    }

    /**
     * 指定された基底名、ロケール、クラスローダを使用して、リソースバンドルを取得します。
     * 
     * @param baseName
     *            基底名
     * @param locale
     *            ロケール
     * @param loader
     *            リソースのロード元のクラスローダ
     * @return 指定された基底名とロケールのリソースバンドル
     * @throws MissingResourceException
     *             指定された基底名のリソースバンドルが見つからない場合
     */
    public static CELResourceBundle getBundle(String baseName, Locale locale, ClassLoader loader) {
        return (CELResourceBundle) getXResourceBundleCache().getBundle(baseName, locale, loader);
    }

    /**
     * リソースに使用するデフォルトのクラスローダを返却します。
     * 
     * @return デフォルトのクラスローダ
     */
    static ClassLoader getDefaultLoader() {
        return getXResourceBundleCache().getDefaultLoader();
    }

    /**
     * キャッシュ機能のシングルトンインスタンスを返却します。
     * 
     * @return キャッシュ機能のシングルトンインスタンス
     */
    static XResourceBundleCache getXResourceBundleCache() {
        return CELResourceBundleCacheInstance.instance;
    }

}