/*
 * shohaku
 * Copyright (C) 2005  tomoya nagatani
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package shohaku.core.util;

import java.util.Hashtable;
import java.util.MissingResourceException;

import shohaku.core.helpers.Eval;

/**
 * 拡張可能な階層化リソースバンドルのキャッシュ機能を提供します。
 */
public class XResourceBundleCache {

    /* キャシュの初期容量。 */
    private static final int INITIAL_CACHE_SIZE = 25;

    /* キャシュの負荷係数。 */
    private static final float CACHE_LOAD_FACTOR = (float) 1.0;

    /* バンドルのインスタンスをキャシュします。 */
    private final Hashtable bundleCache = new Hashtable(INITIAL_CACHE_SIZE, CACHE_LOAD_FACTOR);

    /* 候補となるバンドル名の単位でロックを掛ける為のObjectを保管します。 */
    private final Hashtable lockObjects = new Hashtable(INITIAL_CACHE_SIZE, CACHE_LOAD_FACTOR);

    /* 拡張リソースバンドルの生成機能。 */
    private final XResourceBundleCreater bundleCreater;

    /* 拡張リソースバンドルの束縛基準の算出機能。 */
    private final XResourceBundleBaseCalculator bundleBaseCalculator;

    /**
     * 拡張リソースバンドルの生成機能を格納してキャシュ機能を初期化します。
     * 
     * @param creater
     *            拡張リソースバンドルの生成機能
     */
    public XResourceBundleCache(XResourceBundleCreater creater) {
        this.bundleCreater = creater;
        this.bundleBaseCalculator = creater.getBundleBaseCalculator();
    }

    /**
     * 指定された基底名、束縛基準、クラスローダを使用して、拡張リソースバンドルを取得します。
     * 
     * @param baseName
     *            基底名
     * @param bundleBase
     *            束縛基準
     * @param loader
     *            リソースのロード元のクラスローダ
     * @return 指定された基底名と束縛基準の拡張リソースバンドル
     * @throws MissingResourceException
     *             指定された基底名のリソースバンドルが見つからない場合
     */
    public XResourceBundle getBundle(String baseName, Object bundleBase, ClassLoader loader) {
        return getBundleImpl(baseName, bundleBase, loader);
    }

    /**
     * デフォルトのクラスローダを返却します。
     * 
     * @return デフォルトのクラスローダ
     */
    public ClassLoader getDefaultLoader() {
        return this.bundleCreater.getDefaultClassLoader();
    }

    /*
     * private
     */

    /* 拡張リソースバンドルを取得する処理を実装します。 */
    private XResourceBundle getBundleImpl(String baseName, Object bundleBase, ClassLoader loader) {

        if (Eval.isOrNull(baseName, bundleBase, loader)) {
            throw new NullPointerException();
        }

        XResourceBundle bundle = findBundle(loader, bundleBase, baseName);
        if (bundle == null) {
            throw new MissingResourceException("Can't find bundle for base name. " + baseName + ", " + bundleBase,
                    baseName, bundleBase.toString());
        }
        return bundle;
    }

    /* 拡張リソースバンドルを検索して返却します。 拡張リソースバンドルを発見できなかった場合<code>null</code>を返却します。 */
    private XResourceBundle findBundle(ClassLoader loader, Object bundleBase, String baseName) {

        XResourceBundle target = null;

        String[] bundleNames = null;
        Object defaultBundle = bundleBaseCalculator.getDefaultBundleBase();
        if (!Eval.isEquals(defaultBundle, bundleBase)) {
            bundleNames = bundleBaseCalculator.candidateHierarchyBundleNames(baseName, bundleBase);
            target = loadBundle(loader, baseName, bundleNames);
        }
        if (target == null) {
            bundleNames = bundleBaseCalculator.candidateHierarchyBundleNames(baseName, defaultBundle);
            target = loadBundle(loader, baseName, bundleNames);
            if (target == null) {
                target = createBundle(loader, baseName, baseName, null);
            }
        }

        return target;
    }

    /* バンドル名の候補の階層順にリソースを読み取り処理対象となるリソースを返却します、発見できなかった場合<code>null</code>を返却します。 */
    private XResourceBundle loadBundle(ClassLoader loader, String baseName, String[] bundleNames) {

        XResourceBundle parent = null;
        XResourceBundle target = null;
        XResourceBundle bundle = null;

        for (int i = 0; i < bundleNames.length; i++) {
            String bundleName = bundleNames[i];
            if (bundleName == null) {
                continue;
            }
            bundle = createBundle(loader, baseName, bundleName, parent);
            if (bundle != null) {
                parent = bundle;
                if (i != 0) {
                    target = bundle;
                }
            }
        }

        return target;
    }

    /* 個々のバンドル名の候補からリソースを検索して返却します、発見できなかった場合<code>null</code>を返却します。 */
    private XResourceBundle createBundle(ClassLoader loader, String baseName, String bundleName, XResourceBundle parent) {

        XResourceBundle bundle = null;

        // 候補となるバンドル名単位でロックを行う
        synchronized (getLockObject(bundleName)) {
            bundle = (XResourceBundle) findBundleInCache(bundleName);

            if (bundle == null) {
                // バンドル名から束縛基準を算出します。
                Object bundleBase = bundleBaseCalculator.calculateBundleBase(baseName, bundleName);
                // return XResourceBundle or null
                bundle = bundleCreater.createBundle(loader, parent, bundleBase, baseName, bundleName);
                if (bundle != null) {
                    putBundleInCache(bundleName, bundle);
                }
            }
        }

        return bundle;
    }

    /* リソース単位でロックを掛ける為の Object を返却します。 */
    private Object getLockObject(String bundleName) {
        synchronized (lockObjects) {
            Object key = lockObjects.get(bundleName);
            if (key == null) {
                key = new Object();
                lockObjects.put(bundleName, key);
            }
            return key;
        }
    }

    /* キャシュからバンドル名の示す XResourceBundle を返却します。 */
    private Object findBundleInCache(String bundleName) {
        synchronized (bundleCache) {
            return bundleCache.get(bundleName);
        }
    }

    /* バンドル名の示す XResourceBundle をキャシュします。 */
    private void putBundleInCache(String bundleName, Object value) {
        synchronized (bundleCache) {
            bundleCache.put(bundleName, value);
        }
    }

}
