/*
 * shohaku
 * Copyright (C) 2005  tomoya nagatani
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package shohaku.core.util.cel;

import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import shohaku.core.helpers.Eval;
import shohaku.core.helpers.Seek;

/**
 * 合成式言語 <code>CEL(Composite CELExpression Language)</code> のユーティリティメソッドを提供します。
 */
public class CELHelper {

    /**
     * 引数の書式として解析してリストデータとして結果オブジェクトに格納して返却します。
     * 
     * @param binder
     *            処理基のバインダー
     * @param exp
     *            式
     * @param beginIndex
     *            開始インデックス
     * @param context
     *            引数値の一覧
     * @return 引数を格納した結果オブジェクト
     */
    public static CELReturnValue getArguments(CELBinder binder, String exp, int beginIndex, CELContext context) {
        CELReturnValue rValue = getArrayListReturnValue(binder, exp, beginIndex, context, '(', ')');
        List list = (List) rValue.value;
        return new CELReturnValue(list.toArray(), rValue.nextIndex);
    }

    /**
     * リスト書式として解析して普遍リストデータとして結果オブジェクトに格納して返却します。
     * 
     * @param binder
     *            処理基のバインダー
     * @param exp
     *            式
     * @param beginIndex
     *            開始インデックス
     * @param context
     *            引数値の一覧
     * @return 普遍リストを格納した結果オブジェクト
     */
    public static CELReturnValue getUnmodifiableList(CELBinder binder, String exp, int beginIndex, CELContext context) {
        CELReturnValue rValue = getArrayListReturnValue(binder, exp, beginIndex, context, '[', ']');
        List list = Collections.unmodifiableList((List) rValue.value);
        return new CELReturnValue(list, rValue.nextIndex);
    }

    /**
     * セット書式として解析して普遍セットデータとして結果オブジェクトに格納して返却します。
     * 
     * @param binder
     *            処理基のバインダー
     * @param exp
     *            式
     * @param beginIndex
     *            開始インデックス
     * @param context
     *            引数値の一覧
     * @return 普遍セットを格納した結果オブジェクト
     */
    public static CELReturnValue getUnmodifiableSet(CELBinder binder, String exp, int beginIndex, CELContext context) {
        CELReturnValue rValue = getLinkedHashSetReturnValue(binder, exp, beginIndex, context, '<', '>');
        Set set = Collections.unmodifiableSet((Set) rValue.value);
        return new CELReturnValue(set, rValue.nextIndex);
    }

    /**
     * リスト書式として解析して<code>java.util.ArrayList</code>データとして結果オブジェクトに格納して返却します。
     * 
     * @param binder
     *            処理基のバインダー
     * @param exp
     *            式
     * @param beginIndex
     *            開始インデックス
     * @param context
     *            引数値の一覧
     * @param beginLiteral
     *            リストの開始リテラル
     * @param endLiteral
     *            リストの終了リテラル
     * @return リストを格納した結果オブジェクト
     */
    public static CELReturnValue getArrayListReturnValue(CELBinder binder, String exp, int beginIndex,
            CELContext context, char beginLiteral, char endLiteral) {

        if (!Eval.isCharAt(exp, beginIndex, beginLiteral)) {
            throw new CELFormatException("format err :" + exp);
        }
        ArrayList list = new ArrayList();
        int off = ++beginIndex;
        if (!Eval.isCharAt(exp, off, endLiteral)) {
            while (off < exp.length()) {
                off = Seek.skipSp(exp, off);
                CELReturnValue rValue = binder.getReturnValue(exp, off, context);
                list.add(rValue.value);
                off = Seek.skipSp(exp, rValue.nextIndex);
                if (off >= exp.length() || Eval.isCharAt(exp, off, endLiteral)) {
                    break;
                }
                if (!Eval.isCharAt(exp, off, ',')) {
                    throw new CELFormatException("format err :" + exp);
                }
                off = Seek.skipSp(exp, ++off);
            }
            off = Seek.skipSp(exp, off);
            if (!Eval.isCharAt(exp, off, endLiteral)) {
                throw new CELFormatException("format err :" + exp);
            }
            if (list.isEmpty()) {
                throw new CELFormatException("format err :" + exp);
            }
        }
        return new CELReturnValue(list, ++off);// next index

    }

    /**
     * セット書式として解析して<code>java.util.LinkedHashSet</code>データとして結果オブジェクトに格納して返却します。
     * 
     * @param binder
     *            処理基のバインダー
     * @param exp
     *            式
     * @param beginIndex
     *            開始インデックス
     * @param context
     *            引数値の一覧
     * @param beginLiteral
     *            リストの開始リテラル
     * @param endLiteral
     *            リストの終了リテラル
     * @return セットを格納した結果オブジェクト
     */
    public static CELReturnValue getLinkedHashSetReturnValue(CELBinder binder, String exp, int beginIndex,
            CELContext context, char beginLiteral, char endLiteral) {

        if (!Eval.isCharAt(exp, beginIndex, beginLiteral)) {
            throw new CELFormatException("format err :" + exp);
        }
        LinkedHashSet set = new LinkedHashSet();
        int off = ++beginIndex;
        if (!Eval.isCharAt(exp, off, endLiteral)) {
            while (off < exp.length()) {
                off = Seek.skipSp(exp, off);
                CELReturnValue rValue = binder.getReturnValue(exp, off, context);
                set.add(rValue.value);
                off = Seek.skipSp(exp, rValue.nextIndex);
                if (off >= exp.length() || Eval.isCharAt(exp, off, endLiteral)) {
                    break;
                }
                if (!Eval.isCharAt(exp, off, ',')) {
                    throw new CELFormatException("format err :" + exp);
                }
                off = Seek.skipSp(exp, ++off);
            }
            off = Seek.skipSp(exp, off);
            if (!Eval.isCharAt(exp, off, endLiteral)) {
                throw new CELFormatException("format err :" + exp);
            }
            if (set.isEmpty()) {
                throw new CELFormatException("format err :" + exp);
            }
        }
        return new CELReturnValue(set, ++off);// next index

    }

    /**
     * マップ書式として解析して普遍マップデータとして結果オブジェクトに格納して返却します。
     * 
     * @param binder
     *            処理基のバインダー
     * @param exp
     *            式
     * @param beginIndex
     *            開始インデックス
     * @param context
     *            引数値の一覧
     * @return 普遍マップを格納した結果オブジェクト
     */
    public static CELReturnValue getUnmodifiableMap(CELBinder binder, String exp, int beginIndex, CELContext context) {
        CELReturnValue rValue = getLinkedHashMapReturnValue(binder, exp, beginIndex, context, '{', '}');
        Map map = Collections.unmodifiableMap((Map) rValue.value);
        return new CELReturnValue(map, rValue.nextIndex);
    }

    /**
     * マップ書式として解析して<code>java.util.LinkedHashMap</code>データとして結果オブジェクトに格納して返却します。
     * 
     * @param binder
     *            処理基のバインダー
     * @param exp
     *            式
     * @param beginIndex
     *            開始インデックス
     * @param context
     *            引数値の一覧
     * @param beginLiteral
     *            リストの開始リテラル
     * @param endLiteral
     *            リストの終了リテラル
     * @return マップを格納した結果オブジェクト
     */
    public static CELReturnValue getLinkedHashMapReturnValue(CELBinder binder, String exp, int beginIndex,
            CELContext context, char beginLiteral, char endLiteral) {

        if (!Eval.isCharAt(exp, beginIndex, beginLiteral)) {
            throw new CELFormatException("format err :" + exp);
        }
        LinkedHashMap map = new LinkedHashMap();
        int off = ++beginIndex;
        if (!Eval.isCharAt(exp, off, endLiteral)) {
            while (off < exp.length()) {
                off = Seek.skipSp(exp, off);
                // get key
                CELReturnValue rKey = binder.getReturnValue(exp, off, context);
                off = Seek.skipSp(exp, rKey.nextIndex);
                if (off >= exp.length()) {
                    throw new CELFormatException("format err :" + exp);
                }
                // key value separator
                if (!Eval.isCharAt(exp, off, '=')) {
                    throw new CELFormatException("format err :" + exp);
                }
                off = Seek.skipSp(exp, ++off);
                // get value
                CELReturnValue rValue = binder.getReturnValue(exp, off, context);
                off = Seek.skipSp(exp, rValue.nextIndex);
                // add Entry
                map.put(rKey.value, rValue.value);
                if (off >= exp.length() || Eval.isCharAt(exp, off, endLiteral)) {
                    break;
                }
                // map entry separator
                if (!Eval.isCharAt(exp, off, ',')) {
                    throw new CELFormatException("format err :" + exp);
                }
                off = Seek.skipSp(exp, ++off);
            }
            off = Seek.skipSp(exp, off);
            if (!Eval.isCharAt(exp, off, endLiteral)) {
                throw new CELFormatException("format err :" + exp);
            }
            if (map.isEmpty()) {
                throw new CELFormatException("format err :" + exp);
            }
        }
        return new CELReturnValue(map, ++off);// next index

    }

    /**
     * 式の終端のインデックスを検索して返却します。 発見出来ない場合は -1 を返却します。
     * 
     * @param expression
     *            式
     * @param beginIndex
     *            開始インデックス
     * @param endLiteral
     *            終了リテラル文字
     * @return 式の終端のインデックス
     */
    public static int findNextIndex(String expression, int beginIndex, char endLiteral) {
        int off = ++beginIndex;// begin literal length == 1
        while (off < expression.length()) {
            off = expression.indexOf(endLiteral, off);
            if (-1 == off) {
                return -1;
            }
            if (0 == off || (0 < off && '\\' != expression.charAt(off - 1))) {
                break;
            }
            off++;
        }
        return (off + 1);// end to next
    }

}
