/*
 * shohaku
 * Copyright (C) 2005  tomoya nagatani
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package shohaku.core.util.cel;

import java.lang.reflect.Array;
import java.util.List;
import java.util.Map;

import shohaku.core.beans.BeanUtilities;
import shohaku.core.beans.InvocationBeansException;
import shohaku.core.helpers.Cutter;
import shohaku.core.helpers.Eval;
import shohaku.core.helpers.Seek;

/**
 * リフレクションを使用してオブジェクトへアクセスする式を提供します。
 */
public class IntrospectExpression implements CELExpression {

    /* この式のリテラル文字を定義します。 */
    private static final CELLiteral literal = new CELLiteral('%', '%');

    /**
     * リテラル文字 '%', '%' を返却します。
     * 
     * @return リテラル文字
     */
    public CELLiteral getLiteral() {
        return literal;
    }

    /**
     * 式を実行して値を返却します。
     * 
     * @param binder
     *            処理基のバインダー
     * @param expression
     *            式
     * @param beginIndex
     *            開始インデックス
     * @param context
     *            引数値の一覧
     * @return 式の結果
     */
    public CELReturnValue execute(CELBinder binder, String expression, int beginIndex, CELContext context) {
        int off = ++beginIndex;
        String exp = expression;
        Object o = null;

        // start value
        off = Seek.skipSp(exp, off);
        CELReturnValue stRValue = binder.getReturnValue(exp, off, context);
        o = stRValue.value;
        off = stRValue.nextIndex;

        off = Seek.skipSp(exp, off);

        // index, Field, Method
        try {
            while (off < exp.length()) {
                int _skipOff = Seek.skipSp(exp, off);
                if (Eval.isCharAt(exp, _skipOff, literal.end)) {
                    break;
                }
                if (Eval.isCharAt(exp, off, '[')) {
                    String key = Cutter.cutIndexOf(exp, ++off, ']');
                    if (o instanceof Map) {
                        o = ((Map) o).get(key);
                    } else {
                        int index = Integer.parseInt(key);
                        if (o instanceof List) {
                            o = ((List) o).get(index);
                        } else if (Eval.isArray(o)) {
                            o = Array.get(o, index);
                        } else {
                            throw new CELFormatException("format err :" + exp);
                        }
                    }
                    off += key.length() + 1;
                } else {
                    if (Eval.isCharAt(exp, off, '.')) {
                        off++;
                    }
                    int end = Seek.orIndexOf(exp, "[(.#%".toCharArray(), off);
                    if (-1 < end) {
                        if (Eval.isCharAt(exp, end, '(')) {// method 'method(arg...)'
                            String methodName = exp.substring(off, end);
                            CELReturnValue args = CELHelper.getArguments(binder, exp, end, context);
                            Class c = (o instanceof Class) ? (Class) o : o.getClass();
                            o = BeanUtilities.invokeMethod(c, o, methodName, (Object[]) args.value);
                            off = args.nextIndex;
                        } else if (Eval.isCharAt(exp, end, '#')) {// field '#field'
                            String fieldName = exp.substring(off, end);
                            o = BeanUtilities.getFieldValue(o, fieldName);
                            off = end;
                        } else {// property 'property. or property[num] or property%'
                            String propertyName = exp.substring(off, end);
                            o = BeanUtilities.getProperty(o, propertyName);
                            off = end;
                        }
                    } else {
                        throw new CELFormatException("format err :" + exp);
                    }
                }
            }
        } catch (InvocationBeansException e) {
            throw new CELFormatException("format err :" + exp, e);
        } catch (IndexOutOfBoundsException e) {
            throw new CELFormatException("format err :" + exp, e);
        } catch (NumberFormatException e) {
            throw new CELFormatException("format err :" + exp, e);
        }

        off = Seek.skipSp(exp, off);

        if (!Eval.isCharAt(exp, off, literal.end)) {
            throw new CELFormatException("format err :" + exp);
        }
        return new CELReturnValue(o, ++off);
    }

}